/* $Id: lapack.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_LAPACK_INCLUDED
#define GENLIB2_LAPACK_INCLUDED

#include "vector.h"
#include "matrix.h"

// *** FORTRAN subroutine names maybe depend on the system. ***
#ifdef GENLIB2_HAVE_INTEL_MKL
	// Intel Math Kernel Library (MKL)
	#define F77_DPOTRF DPOTRF
	#define F77_DPOTRS DPOTRS
	#define F77_DPOTRI DPOTRI
	#define F77_DPOSV  DPOSV
#else
	// original LAPACK/BLAS source complied with g77 (default)
	#define F77_DPOTRF dpotrf_
	#define F77_DPOTRS dpotrs_
	#define F77_DPOTRI dpotri_
	#define F77_DPOSV  dposv_
#endif

// external FORTRAN subroutines
extern "C"
{
	// factorize a symmetric positive definite matrix
	void F77_DPOTRF(char&, int&, double&, int&, int&);

	// invert matrix using factorization
	void F77_DPOTRI(char&, int&, double&, int&, int&);

	// solving system using factorization
	void F77_DPOTRS(char&, int&, int&, double&, int&, double&, int&, int&);

	// solving system of linear equations
	void F77_DPOSV(char&, int&, int&, double&, int&, double&, int&, int&);
}


namespace GenLib2
{

/** Cholesky decomposition.
 *  Computes the Cholesky factorization of a symmetric positive
 *  definite matrix.
 */
int lapack_dpotrf(Matrix<double>& A)
{
	char uplo = 'l';
	int n = A.rows();
	int lda = A.rows();
	int info = 0;

	F77_DPOTRF(uplo, n, A(0,0), lda, info);

	return info;
}


/** Matrix inversion.
 *  Computes the inverse of a symmetric positive definite matrix,
 *  using the Cholesky factorization computed by lapack_dpotrf.
 */
int lapack_dpotri(Matrix<double>& A)
{
	char uplo = 'l';
	int n = A.rows();
	int lda = A.rows();
	int info = 0;

	F77_DPOTRI(uplo, n, A(0,0), lda, info);

	return info;
}


/** Solve linear equation system.
 *  Solving a symmetric positive definite linear equation system,
 *  using the Cholesky factorization computed by lapack_dpotrf.
 */
int lapack_dpotrs(Matrix<double>& A, Vector<double>& b)
{
	char uplo = 'l';
	int n = A.rows();
	int nrhs = 1;
	int lda = A.rows();
	int ldb = b.size();
	int info = 0;

	F77_DPOTRS(uplo, n, nrhs, A(0,0), lda, b(0), ldb, info);

	return info;
}


/** Solve linear equation system.
 *  Solving a symmetric positive definite linear equation system.
 */
int lapack_dposv(Matrix<double>& A, Vector<double>& b)
{
	char uplo = 'l';
	int n = A.rows();
	int nrhs = 1;
	int lda = A.rows();
	int ldb = b.size();
	int info = 0;

	F77_DPOSV(uplo, n, nrhs, A(0,0), lda, b(0), ldb, info);
	
	return info;
}

} // namespace GenLib2

#endif /* GENLIB2_LAPACK_INCLUDED */
