/* $Id: vector.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_VECTOR_INCLUDED
#define GENLIB2_VECTOR_INCLUDED

#include <cassert>
#include <iostream>

using namespace std;

namespace GenLib2
{

/** Vector class.
 *  General vector class.
 *  @author Dirk Stallmann
 */
template<class T>
class Vector
{
	// addition and subtraction
	//Vector add(const Vector& a, const Vector& b);
	//Vector sub(const Vector& a, const Vector& b);

	// multiplication
	//Vector mult(const Vector& a, const Vector& b);

	/// dot product
	//friend T dot(const Vector& a, const Vector& b);

protected:
	// internal helper functions
	/// allocate memory
	void allocate(int n);

	/// free memory
	void free();

	/// copy array
	void copy(const T* v);

	/// set all elements to value
	void set(const T& val);

public:
	// constructors/destructor
	/// constructor matrix without size
	Vector();

	/// constructor initilize with value
	Vector(int n, const T& val=T(0));

	/// constructor, initialize with array
	Vector(int n, const T* v);

	/// copy constructor
	Vector(const Vector& A);

	/// destructor
	~Vector();

	// methods
	// copy operator
	Vector& operator=(const Vector& a);

	/// new size
	Vector& newsize(int n);

	/// set all elements to zero
	Vector& clear() { set(T(0)); return *this; }

	// assign scalar
	Vector& operator=(const T& v);

	// member data access
	/// vector size
	int dim() const { return _n; }
	int size() const { return _n; }

	// data check
	bool is_data() const { return (_data != NULL); }

	/// set and get an matrix element without boundary check
	T& operator()(int i) { return _data[i]; }
	const T& operator()(int i) const { return _data[i]; }

	// I/O
	/// read matrix
	void read(istream& is);

	/// print matrix
	void print(ostream& os) const; 

	// composite assignment operators...
	//
	// multiplication with a scalar
	Vector& operator*=(T s);

	/// division by a scalar
	Vector& operator/=(T s);

protected:
	/// vector dimension
	int _n;

	/// vector elements
	T* _data;
};



// allocation

template<class T>
void Vector<T>::allocate(int n)
{
	assert(_data == NULL); // to avoid memory leaks

	// allocate vector
	_data = (T*)new T[_n=n];
	assert(_data != NULL);
}

template<class T>
void Vector<T>::free()
{
	// do nothing, if no memory has been previously allocated
	if (_data == NULL) return;

	// free previously allocated vector
	if (_data != NULL) delete[] _data;

	_data = NULL;
}


// copy array

template<class T>
void Vector<T>::copy(const T* v)
{
	assert(_data != NULL);
	for (int i = 0; i < _n; i++)
		_data[i] = v[i];
}


// set all elements to value

template<class T>
void Vector<T>::set(const T& val)
{
	//assert(_data != NULL);
	if(_data == NULL) return; // nothing to do!
	for (int i = 0; i < _n; i++)
		_data[i] = val;
}


// constructor without size (default)

template<class T>
Vector<T>::Vector() : _n(0), _data(NULL) { }


// constructor with size and initilize with value

template<class T>
Vector<T>::Vector(int n, const T& val) : _n(0), _data(NULL)
{
	allocate(n);
	set(val);
}


// constructor, initialize with array

template<class T>
Vector<T>::Vector(int n, const T* v) : _n(0), _data(NULL)
{
	allocate(n);
	copy(v);
}


// copy constructor

template<class T>
Vector<T>::Vector(const Vector& A) : _n(0), _data(NULL)
{
	allocate(A._n);
	copy(A._data);
}


// destructor

template<class T>
Vector<T>::~Vector()
{
	free();
}


// assignment, copy data

template<class T>
Vector<T>& Vector<T>::operator=(const Vector& a)
{
	if (_data == a._data) return *this;
		
	if (_n == a._n)
		copy(a._data);
	else
		{
		free();
		allocate(a._n);
		copy(a._data);
		}

	return *this;
}


// new size

template<class T>
Vector<T>& Vector<T>::newsize(int n)
{
	if (_n == n) return *this;
	if (_data != NULL) free();
	allocate(n);
	return *this;
}


// assign scalar

template<class T>
Vector<T>& Vector<T>::operator=(const T& v)
{
	set(v);
	return *this;
}


// read vector

template<class T>
void Vector<T>::read(istream& is)
{
	int n = 0;
	is >> n;
	newsize(n);

	for (int i = 0; i < _n; i++)
		is >> _data[i];
}


// print vector

template<class T>
void Vector<T>::print(ostream& os) const
{
	os << _n << endl;
	for (int i = 0; i < _n; i++)
		os << _data[i] << " ";
	os << endl;
}


// multiplication with a scalar

template<class T>
Vector<T>& Vector<T>::operator*=(T s)
{
	for (int i = 0; i < _n; i++) _data[i] *= s;
		return *this;
}


// division by a scalar

template<class T>
Vector<T>& Vector<T>::operator/=(T s)
{
	for (int i = 0; i < _n; i++) _data[i] /= s;
		return *this;
}


// addition
/*
template<class T>
void Vector<T>::add(const Vector<T>& a, const Vector<T>& b)
{
	assert(a._rows == b._rows);
	assert(a._cols == b._cols);

	adjust(a._rows, a._cols);
//
	register T* pa = a._data[0];
	register T* pb = b._data[0];
	register T* pc = _data[0];
	register T* end = _data[0] + _rows * _cols;
//

	while (pc < end)
		*pc++ = *pa++ + *pb++;

	for (int i = 0; i < _rows; i++)
		for (int j = 0; j < _cols; j++)
			_data[i][j] = a._data[i][j] + b._data[i][j];
}
*/


// subtraction
/*
template<class T>
void Vector<T>::sub(const Vector<T>& a, const Vector<T>& b)
{
	assert(a._rows == b._rows);
	assert(a._cols == b._cols);

	adjust(a._rows, a._cols);
//
	register T* pa = a._data[0];
	register T* pb = b._data[0];
	register T* pc = _data[0];
	register T* end = _data[0] + _rows * _cols;

	while (pc < end)
		*pc++ = *pa++ - *pb++;
//
	for (int i = 0; i < _rows; i++)
		for (int j = 0; j < _cols; j++)
			_data[i][j] = a._data[i][j] - b._data[i][j];
}
*/


/// non-member functions...

/// I/O operators

template<class T>
std::istream& operator>>(std::istream& is, Vector<T>& a)
{
	a.read(is);
	return is;
}

template<class T>
std::ostream& operator<<(std::ostream& os, const Vector<T>& a)
{
	a.print(os);
	return os;
}


/// basic matrix algorithms

/// addition
template<class T>
Vector<T> operator+(const Vector<T>& a, const Vector<T>& b)
{
	int n = a.size();

	assert(n == b.size());

    Vector<T> c(n);
	
	for (register int i = 0; i < n; i++)
			c(i) = a(i) + b(i);

	return c;
}


/// subtraction
template<class T>
Vector<T> operator-(const Vector<T>& a, const Vector<T>& b)
{
	int n = a.size();

	assert(n == b.size());

    Vector<T> c(n);
	
	for (int i = 0; i < n; i++)
			c(i) = a(i) - b(i);

	return c;
}
		

/// dot product
template<class T>
T dot_prod(const Vector<T>& a, const Vector<T>& b)
{
	//assert(a._n == b._n);

	//register T sum = T(0);
	//register T *pa = a._data, *pb = b._data;

	//for (register int i = a._n; i > 0; --i)
	//	sum += *pa++ * *pb++;
	
	int n = a.size();

	assert(n == b.size());

	T sum = 0;

	for (int i = 0; i < n; i++)
		sum += a(i) * b(i);

	return sum;
}

} // namespace GenLib2

#endif /* GENLIB2_VECTOR_INCLUDED */
